# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - ImprovedTableParser library

    @copyright: 2012 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import wikiutil
from shlex import shlex
from StringIO import StringIO
import re

# Regular expressions.

syntax = {
    # At start of line:
    "rows"      : (r"^==",                  re.MULTILINE),  # ==
    # Within text:
    "columns"   : (r"\|\|[ \t]*",           0),             # || whitespace
    # At start of column text:
    "column"    : (r"^\s*<(.*?)>\s*(.*)",   re.DOTALL),     # whitespace < attributes > whitespace
    }

patterns = {}
for name, (value, flags) in syntax.items():
    patterns[name] = re.compile(value, re.UNICODE | flags)

# Functions.

def parse(s):

    "Parse 's', returning a table definition."

    rows = []
    table_attrs = {}

    # Extract each row from the definition.

    for row_text in patterns["rows"].split(s):
        columns = []

        # Extract each column from the row.

        for text in patterns["columns"].split(row_text):

            # Extract the attribute and text sections.

            match = patterns["column"].search(text)
            if match:
                attribute_text, text = match.groups()
                columns.append((parseAttributes(attribute_text, True), text))
            else:
                columns.append(({}, text))

        # Extract row- and table-level attributes.

        row_attrs = {}

        if columns:
            attrs, column = columns[0]

            for name, value in attrs.items():
                if name.startswith("row"):
                    row_attrs[name] = value
                    del attrs[name]
                elif name.startswith("table"):
                    table_attrs[name] = value
                    del attrs[name]

        rows.append((row_attrs, columns))

    return table_attrs, rows

def parseAttributes(s, escape=True):

    """
    Parse the table attributes string 's', returning a mapping of names to
    values. If 'escape' is set to a true value, the attributes will be suitable
    for use with the formatter API.
    """

    attrs = {}
    f = StringIO(s)
    name = None
    need_value = False

    for token in shlex(f):

        # Capture the name if needed.

        if name is None:
            name = escape and wikiutil.escape(token) or token

        # Detect either an equals sign or another name.

        elif not need_value:
            if token == "=":
                need_value = True
            else:
                attrs[name.lower()] = escape and "true" or True
                name = wikiutil.escape(token)

        # Otherwise, capture a value.

        else:
            # Quoting of attributes done similarly to parseAttributes.

            if escape and token:
                if token[0] in ("'", '"'):
                    token = wikiutil.escape(token)
                else:
                    token = '"%s"' % wikiutil.escape(token, 1)
    
            attrs[name.lower()] = token
            name = None
            need_value = False

    return attrs

# Formatting of embedded content.
# NOTE: Borrowed from EventAggregator.

def getParserClass(request, format):

    """
    Return a parser class using the 'request' for the given 'format', returning
    a plain text parser if no parser can be found for the specified 'format'.
    """

    try:
        return wikiutil.searchAndImportPlugin(request.cfg, "parser", format or "plain")
    except wikiutil.PluginMissingError:
        return wikiutil.searchAndImportPlugin(request.cfg, "parser", "plain")

def formatText(text, request, fmt):

    "Format the given 'text' using the specified 'request' and formatter 'fmt'."

    parser_cls = getParserClass(request, request.page.pi["format"])
    parser = parser_cls(text, request, line_anchors=False)
    return request.redirectedOutput(parser.format, fmt, inhibit_p=True)

# Common formatting functions.

def formatTable(text, request, fmt):

    "Format the given 'text' using the specified 'request' and formatter 'fmt'."

    attrs, table = parse(text)

    request.write(fmt.table(1, attrs))

    for row_attrs, columns in table:
        request.write(fmt.table_row(1, row_attrs))

        for column_attrs, column_text in columns:
            request.write(fmt.table_cell(1, column_attrs))
            request.write(formatText(column_text, request, fmt))
            request.write(fmt.table_cell(0))

        request.write(fmt.table_row(0))

    request.write(fmt.table(0))

# vim: tabstop=4 expandtab shiftwidth=4
